@@ -77,6 +77,14 @@
 
 #include "ns_ioalib_impl.h"
 
+#include <openssl/aes.h>
+#include <openssl/err.h>
+#include <openssl/pem.h>
+
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+  #include <openssl/modes.h>
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -198,6 +206,9 @@ typedef struct _turn_params_ {
   int no_tls;
   int no_dtls;
 
+  struct event *tls_ctx_update_ev;
+  pthread_mutex_t tls_mutex;
+
 //////////////// Common params ////////////////////
 
   int verbose;
@@ -306,6 +317,11 @@ typedef struct _turn_params_ {
 
   unsigned long cpus;
 
+  ///////// Encryption /////////
+  char secret_key_file[1025];
+  unsigned char secret_key[1025];
+  int keep_address_family;
+
 } turn_params_t;
 
 extern turn_params_t turn_params;
@@ -354,6 +370,24 @@ void set_bps_capacity(band_limit_t value);
 band_limit_t get_max_bps(void);
 void set_max_bps(band_limit_t value);
 
+///////// AES ENCRYPTION AND DECRYPTION ////////
+
+struct ctr_state {
+	unsigned char ivec[16];
+	unsigned int num;
+	unsigned char ecount[16];
+};
+void generate_aes_128_key(char* filePath, unsigned char* returnedKey);
+unsigned char *base64encode (const void *b64_encode_this, int encode_this_many_bytes);
+void encrypt_aes_128(unsigned char* in, const unsigned char* mykey);
+unsigned char *base64decode (const void *b64_decode_this, int decode_this_many_bytes);
+void decrypt_aes_128(char* in, const unsigned char* mykey);
+int decodedTextSize(char *input);
+char* decryptPassword(char* in, const unsigned char* mykey);
+int init_ctr(struct ctr_state *state, const unsigned char iv[8]);
+
+
+
 ///////////////////////////////
 
 #ifdef __cplusplus
