@@ -172,6 +172,9 @@ struct _turn_turnserver {
 
 	/* HTTP-Admin-Server: */
 	int use_http;
+
+	/* Keep Address Family */
+	int keep_address_family;
 };
 
 const char * get_version(turn_turnserver *server);
@@ -216,7 +219,8 @@ void init_turn_server(turn_turnserver* server,
 				    allocate_bps_cb allocate_bps_func,
 				    int oauth,
 				    const char* oauth_server_name,
-				    int use_http);
+				    int use_http,
+					int keep_address_family);
 
 ioa_engine_handle turn_server_get_engine(turn_turnserver *s);
 
